==> libcli-0.0.4.tar.gz: lib/command/command_service.dart <==
import 'dart:typed_data';
import 'package:dio/dio.dart';
import 'package:rxdart/rxdart.dart';
import 'proto_object.dart';

/// communicate with server with command using ajax,protobuf and command pattern
/// it basically send a action command and receive response command
///
/// For example:
///
///     CommandService commandService = CommandService('http://localhost',  MapXXX());
abstract class CommandService {
  /// remote service url
  String url;

  /// implement http post using DIO
  final Dio _dio = Dio();

  /// find object by id
  ProtoObject findObjectById(int id, List<int> bytes);

  /// valid id range
  int getIdValidRange();

  /// use mock response instead of using dio to get real response
  ///
  /// For example:
  ///
  ///     CommandService fakeService = CommandService('', null);
  ///     StringResponse r = StringResponse();
  ///     r.text = "fake";
  ///     fakeService.mockResponse = r;
  ProtoObject mockResponse;

  /// use mock responder to respond
  ///
  /// For example:
  ///
  ///      CommandService fakeService = CommandService('', null);
  ///      fakeService.mockResponder = (cmd) {
  ///      var r = StringResponse();
  ///      r.text = "fake";
  ///      return r;
  ///       };
  Function(ProtoObject cmd) mockResponder;

  /// set connect timeout in ms, default 5000ms
  ///
  /// For example:
  ///
  ///     commandService.timeout = 5000; // 5 seconds
  int timeout = 5000;

  CommandService() {
    _dio.interceptors
        .add(InterceptorsWrapper(onRequest: (RequestOptions options) {
      // 在发送请求之前做一些预处理
      //我这边是在发送前到SharedPreferences（本地存储）中取出token的值，然后添加到请求头中
      //dio.lock()是先锁定请求不发送出去，当整个取值添加到请求头后再dio.unlock()解锁发送出去
      /*
                _dio.lock();
                Future<dynamic> future = Future(()async{
                    SharedPreferences prefs =await SharedPreferences.getInstance();
                    return prefs.getString("loginToken");
                });
                return future.then((value) {
                    options.headers["Authorization"] = value;
                    return options;
                }).whenComplete(() => _dio.unlock()); // unlock the dio
                */
    }, onResponse: (Response response) {
      // 在返回响应数据之前做一些预处理
      return response; // continue
    }, onError: (DioError e) {
      // 当请求失败时做一些预处理
      return e; //continue
    }));
  }

  /// set mock adapter to dio
  ///
  /// For example:
  ///
  ///     DioMock mock = DioMock();
  ///     mock.stringResponse = "Hello World";
  ///     CommandService mockService = CommandService('', null);
  ///     mockService.setMockHttpAdapter(mock);
  void setMockHttpAdapter(HttpClientAdapter adapter) {
    this._dio.httpClientAdapter = adapter;
  }

  /// encode protobuf object into bytes
  ///
  /// For example:
  ///
  ///     EchoAction echoAction = EchoAction();
  ///     echoAction.text = 'helloWorld';
  ///     List<int> list = commandService.encode(echoAction);
  ///     expect(list.length, 6);
  Uint8List encode(ProtoObject obj) {
    Uint8List bytes = obj.writeToBuffer();
    Uint8List list = Uint8List(bytes.length + 2);
    list.setRange(0, bytes.length, bytes);
    int id = obj.mapIdXXX();
    list[list.length - 2] = id & 0xff;
    list[list.length - 1] = (id >> 8) & 0xff;
    return list;
  }

  /// decode bytes array to protobuf object
  ///
  /// For example:
  ///
  ///     EchoAction decodeAction = commandService.decode(bytes);
  ///     expect(decodeAction.text, 'helloWorld');
  ProtoObject decode(List<int> bytes) {
    List<int> protoBytes = bytes.sublist(0, bytes.length - 2);
    Uint8List idBytes =
        Uint8List.fromList(bytes.sublist(bytes.length - 2, bytes.length));
    final id = idBytes.buffer.asByteData().getInt16(0, Endian.little);
    final max = getIdValidRange();
    if (id < 0 || id > max) {
      throw new Exception('id $id out of range , the max id allowed is $max');
    }
    ProtoObject obj = findObjectById(id, protoBytes);
    return obj;
  }

  /// do http post to remote url
  ///
  /// For example:
  ///
  ///     EchoAction decodeAction = commandService.decode(bytes);
  ///     expect(decodeAction.text, 'helloWorld');
  Future<List<int>> post(Uint8List bytes) async {
    var response = await _dio.post(url,
        data: Stream.fromIterable(bytes.map((e) => [e])),
        options: Options(
            responseType: ResponseType.bytes,
            headers: {'Content-Type': 'multipart/form-data', 'accept': ''}));
    return response.data;
  }

  /// execute command and return result
  ///
  /// For example:
  ///
  ///     EchoAction echoAction = new EchoAction();
  ///     echoAction.text = 'helloWorld';
  ///     commandService.execute(echoAction).listen((result) {
  ///     expect(result != null, true);
  ///     });
  Observable<ProtoObject> execute(ProtoObject obj) {
    assert(url != null && url.length > 0);

    _dio.options.connectTimeout = timeout;
    _dio.options.receiveTimeout = timeout;
    _dio.options.sendTimeout = timeout;
    if (this.mockResponse != null) return Observable.just(this.mockResponse);
    if (this.mockResponder != null) return Observable.just(mockResponder(obj));
    Uint8List bytes = this.encode(obj);
    var fromFutureObservable = Observable.fromFuture(this.post(bytes));
    return fromFutureObservable.switchMap<ProtoObject>((List<int> i) {
      return Observable.just(this.decode(i));
    });
  }
}

==> libcli-0.0.5.tar.gz: lib/command/command_service.dart <==
import 'dart:typed_data';
import 'package:dio/dio.dart';
import 'package:rxdart/rxdart.dart';
import 'proto_object.dart';

/// communicate with server with command using ajax,protobuf and command pattern
/// it basically send a action command and receive response command
///
/// For example:
///
///     CommandService commandService = CommandService('http://localhost',  MapXXX());
abstract class CommandService {
  /// remote service url
  String url;

  /// implement http post using DIO
  final Dio _dio = Dio();

  /// find object by id
  ProtoObject newObjectByID(int id, List<int> bytes);

  /// use mock response instead of using dio to get real response
  ///
  /// For example:
  ///
  ///     CommandService fakeService = CommandService('', null);
  ///     StringResponse r = StringResponse();
  ///     r.text = "fake";
  ///     fakeService.mockResponse = r;
  ProtoObject mockResponse;

  /// use mock responder to respond
  ///
  /// For example:
  ///
  ///      CommandService fakeService = CommandService('', null);
  ///      fakeService.mockResponder = (cmd) {
  ///      var r = StringResponse();
  ///      r.text = "fake";
  ///      return r;
  ///       };
  Function(ProtoObject cmd) mockResponder;

  /// set connect timeout in ms, default 5000ms
  ///
  /// For example:
  ///
  ///     commandService.timeout = 5000; // 5 seconds
  int timeout = 5000;

  CommandService() {
    _dio.interceptors
        .add(InterceptorsWrapper(onRequest: (RequestOptions options) {
      // 在发送请求之前做一些预处理
      //我这边是在发送前到SharedPreferences（本地存储）中取出token的值，然后添加到请求头中
      //dio.lock()是先锁定请求不发送出去，当整个取值添加到请求头后再dio.unlock()解锁发送出去
      /*
                _dio.lock();
                Future<dynamic> future = Future(()async{
                    SharedPreferences prefs =await SharedPreferences.getInstance();
                    return prefs.getString("loginToken");
                });
                return future.then((value) {
                    options.headers["Authorization"] = value;
                    return options;
                }).whenComplete(() => _dio.unlock()); // unlock the dio
                */
    }, onResponse: (Response response) {
      // 在返回响应数据之前做一些预处理
      return response; // continue
    }, onError: (DioError e) {
      // 当请求失败时做一些预处理
      return e; //continue
    }));
  }

  /// set mock adapter to dio
  ///
  /// For example:
  ///
  ///     DioMock mock = DioMock();
  ///     mock.stringResponse = "Hello World";
  ///     CommandService mockService = CommandService('', null);
  ///     mockService.setMockHttpAdapter(mock);
  void setMockHttpAdapter(HttpClientAdapter adapter) {
    this._dio.httpClientAdapter = adapter;
  }

  /// encode protobuf object into bytes
  ///
  /// For example:
  ///
  ///     EchoAction echoAction = EchoAction();
  ///     echoAction.text = 'helloWorld';
  ///     List<int> list = commandService.encode(echoAction);
  ///     expect(list.length, 6);
  Uint8List encode(ProtoObject obj) {
    Uint8List bytes = obj.writeToBuffer();
    Uint8List list = Uint8List(bytes.length + 2);
    list.setRange(0, bytes.length, bytes);
    int id = obj.mapIdXXX();
    list[list.length - 2] = id & 0xff;
    list[list.length - 1] = (id >> 8) & 0xff;
    return list;
  }

  /// decode bytes array to protobuf object
  ///
  /// For example:
  ///
  ///     EchoAction decodeAction = commandService.decode(bytes);
  ///     expect(decodeAction.text, 'helloWorld');
  ProtoObject decode(List<int> bytes) {
    List<int> protoBytes = bytes.sublist(0, bytes.length - 2);
    Uint8List idBytes =
        Uint8List.fromList(bytes.sublist(bytes.length - 2, bytes.length));
    final id = idBytes.buffer.asByteData().getInt16(0, Endian.little);
    ProtoObject obj = newObjectByID(id, protoBytes);
    if (obj == null) {
      throw new Exception('response id $id out of range');
    }
    return obj;
  }

  /// do http post to remote url
  ///
  /// For example:
  ///
  ///     EchoAction decodeAction = commandService.decode(bytes);
  ///     expect(decodeAction.text, 'helloWorld');
  Future<List<int>> post(Uint8List bytes) async {
    var response = await _dio.post(url,
        data: Stream.fromIterable(bytes.map((e) => [e])),
        options: Options(
            responseType: ResponseType.bytes,
            headers: {'Content-Type': 'multipart/form-data', 'accept': ''}));
    return response.data;
  }

  /// execute command and return result
  ///
  /// For example:
  ///
  ///     EchoAction echoAction = new EchoAction();
  ///     echoAction.text = 'helloWorld';
  ///     commandService.execute(echoAction).listen((result) {
  ///     expect(result != null, true);
  ///     });
  Observable<ProtoObject> execute(ProtoObject obj) {
    assert(url != null && url.length > 0);

    _dio.options.connectTimeout = timeout;
    _dio.options.receiveTimeout = timeout;
    _dio.options.sendTimeout = timeout;
    if (this.mockResponse != null) return Observable.just(this.mockResponse);
    if (this.mockResponder != null) return Observable.just(mockResponder(obj));
    Uint8List bytes = this.encode(obj);
    var fromFutureObservable = Observable.fromFuture(this.post(bytes));
    return fromFutureObservable.switchMap<ProtoObject>((List<int> i) {
      return Observable.just(this.decode(i));
    });
  }
}
